@@ -16,6 +16,7 @@ package create
 
 import (
 	"bytes"
+	"context"
 	"errors"
 	"fmt"
 	"io"
@@ -23,11 +24,9 @@ import (
 	"path/filepath"
 	"strings"
 
-	"github.com/gohugoio/hugo/hugofs/glob"
-
 	"github.com/gohugoio/hugo/common/hexec"
-	"github.com/gohugoio/hugo/common/hstrings"
 	"github.com/gohugoio/hugo/common/paths"
+	"github.com/gohugoio/hugo/hugofs/hglob"
 
 	"github.com/gohugoio/hugo/hugofs"
 
@@ -37,7 +36,7 @@ import (
 )
 
 const (
-	// DefaultArchetypeTemplateTemplate is the template used in 'hugo new site'
+	// DefaultArchetypeTemplateTemplate is the template used in 'hugo new project'
 	// and the template we use as a fall back.
 	DefaultArchetypeTemplateTemplate = `---
 title: "{{ replace .File.ContentBaseName "-" " " | title }}"
@@ -159,10 +158,10 @@ func (b *contentBuilder) buildDir() error {
 		contentTargetFilenames = append(contentTargetFilenames, abs)
 	}
 
-	var contentInclusionFilter *glob.FilenameFilter
+	var contentInclusionFilter *hglob.FilenameFilter
 	if !b.dirMap.siteUsed {
 		// We don't need to build everything.
-		contentInclusionFilter = glob.NewFilenameFilterForInclusionFunc(func(filename string) bool {
+		contentInclusionFilter = hglob.NewFilenameFilterForInclusionFunc(func(filename string) bool {
 			filename = strings.TrimPrefix(filename, string(os.PathSeparator))
 			for _, cn := range contentTargetFilenames {
 				if strings.Contains(cn, filename) {
@@ -228,10 +227,10 @@ func (b *contentBuilder) buildFile() (string, error) {
 		return "", err
 	}
 
-	var contentInclusionFilter *glob.FilenameFilter
+	var contentInclusionFilter *hglob.FilenameFilter
 	if !usesSite {
 		// We don't need to build everything.
-		contentInclusionFilter = glob.NewFilenameFilterForInclusionFunc(func(filename string) bool {
+		contentInclusionFilter = hglob.NewFilenameFilterForInclusionFunc(func(filename string) bool {
 			filename = strings.TrimPrefix(filename, string(os.PathSeparator))
 			return strings.Contains(contentPlaceholderAbsFilename, filename)
 		})
@@ -291,9 +290,7 @@ func (b *contentBuilder) applyArcheType(contentFilename string, archetypeFi hugo
 func (b *contentBuilder) mapArcheTypeDir() error {
 	var m archetypeMap
 
-	seen := map[hstrings.Tuple]bool{}
-
-	walkFn := func(path string, fim hugofs.FileMetaInfo) error {
+	walkFn := func(ctx context.Context, path string, fim hugofs.FileMetaInfo) error {
 		if fim.IsDir() {
 			return nil
 		}
@@ -301,15 +298,6 @@ func (b *contentBuilder) mapArcheTypeDir() error {
 		pi := fim.Meta().PathInfo
 
 		if pi.IsContent() {
-			pathLang := hstrings.Tuple{First: pi.PathNoIdentifier(), Second: fim.Meta().Lang}
-			if seen[pathLang] {
-				// Duplicate content file, e.g. page.md and page.html.
-				// In the regular build, we will filter out the duplicates, but
-				// for archetype folders these are ambiguous and we need to
-				// fail.
-				return fmt.Errorf("duplicate content file found in archetype folder: %q; having both e.g. %s.md and %s.html is ambigous", path, pi.BaseNameNoIdentifier(), pi.BaseNameNoIdentifier())
-			}
-			seen[pathLang] = true
 			m.contentFiles = append(m.contentFiles, fim)
 			if !m.siteUsed {
 				var err error
