@@ -28,7 +28,7 @@ func TestAttributeExclusion(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark.renderer]
 	unsafe = false
 [markup.goldmark.parser.attribute]
@@ -46,7 +46,7 @@ title: "p1"
 ~~~bash {id="c" onmouseover="alert('code fence')" LINENOS=true}
 foo
 ~~~
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -69,9 +69,9 @@ func TestAttributeExclusionWithRenderHook(t *testing.T) {
 title: "p1"
 ---
 ## Heading {onclick="alert('renderhook')" data-foo="bar"}
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
--- layouts/_default/_markup/render-heading.html --
+-- layouts/_markup/render-heading.html --
 <h{{ .Level }}
   {{- range $k, $v := .Attributes -}}
     {{- printf " %s=%q" $k $v | safeHTMLAttr -}}
@@ -95,7 +95,7 @@ func TestAttributesDefaultRenderer(t *testing.T) {
 title: "p1"
 ---
 ## Heading Attribute Which Needs Escaping { class="a < b" }
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -116,10 +116,10 @@ func TestAttributesHookNoEscape(t *testing.T) {
 title: "p1"
 ---
 ## Heading Attribute Which Needs Escaping { class="Smith & Wesson" }
--- layouts/_default/_markup/render-heading.html --
+-- layouts/_markup/render-heading.html --
 plain: |{{- range $k, $v := .Attributes -}}{{ $k }}: {{ $v }}|{{ end }}|
 safeHTML: |{{- range $k, $v := .Attributes -}}{{ $k }}: {{ $v | safeHTML }}|{{ end }}|
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -136,20 +136,20 @@ func TestLinkInTitle(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 -- content/p1.md --
 ---
 title: "p1"
 ---
 ## Hello [Test](https://example.com)
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
--- layouts/_default/_markup/render-heading.html --
+-- layouts/_markup/render-heading.html --
 <h{{ .Level }} id="{{ .Anchor | safeURL }}">
   {{ .Text }}
   <a class="anchor" href="#{{ .Anchor | safeURL }}">#</a>
 </h{{ .Level }}>
--- layouts/_default/_markup/render-link.html --
+-- layouts/_markup/render-link.html --
 <a href="{{ .Destination | safeURL }}"{{ with .Title}} title="{{ . }}"{{ end }}>{{ .Text }}</a>
 
 `
@@ -165,7 +165,7 @@ func TestHighlight(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup]
 [markup.highlight]
 anchorLineNos = false
@@ -179,7 +179,7 @@ lineNumbersInTable = true
 noClasses = false
 style = 'monokai'
 tabWidth = 4
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 -- content/p1.md --
 ---
@@ -233,15 +233,15 @@ LINE8
 
 func BenchmarkRenderHooks(b *testing.B) {
 	files := `
--- config.toml --
--- layouts/_default/_markup/render-heading.html --
+-- hugo.toml --
+-- layouts/_markup/render-heading.html --
 <h{{ .Level }} id="{{ .Anchor | safeURL }}">
 	{{ .Text }}
 	<a class="anchor" href="#{{ .Anchor | safeURL }}">#</a>
 </h{{ .Level }}>
--- layouts/_default/_markup/render-link.html --
+-- layouts/_markup/render-link.html --
 <a href="{{ .Destination | safeURL }}"{{ with .Title}} title="{{ . }}"{{ end }}>{{ .Text }}</a>
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -278,22 +278,18 @@ D.
 		T:           b,
 		TxtarString: files,
 	}
-	builders := make([]*hugolib.IntegrationTestBuilder, b.N)
 
-	for i := range builders {
-		builders[i] = hugolib.NewIntegrationTestBuilder(cfg)
-	}
-
-	b.ResetTimer()
-
-	for i := 0; i < b.N; i++ {
-		builders[i].Build()
+	for b.Loop() {
+		b.StopTimer()
+		bb := hugolib.NewIntegrationTestBuilder(cfg)
+		b.StartTimer()
+		bb.Build()
 	}
 }
 
 func BenchmarkCodeblocks(b *testing.B) {
 	filesTemplate := `
--- config.toml --
+-- hugo.toml --
 [markup]
   [markup.highlight]
     anchorLineNos = false
@@ -307,7 +303,7 @@ func BenchmarkCodeblocks(b *testing.B) {
     noClasses = true
     style = 'monokai'
     tabWidth = 4
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -337,16 +333,12 @@ FENCE
 			T:           b,
 			TxtarString: files,
 		}
-		builders := make([]*hugolib.IntegrationTestBuilder, b.N)
 
-		for i := range builders {
-			builders[i] = hugolib.NewIntegrationTestBuilder(cfg)
-		}
-
-		b.ResetTimer()
-
-		for i := 0; i < b.N; i++ {
-			builders[i].Build()
+		for b.Loop() {
+			b.StopTimer()
+			bb := hugolib.NewIntegrationTestBuilder(cfg)
+			b.StartTimer()
+			bb.Build()
 		}
 	}
 
@@ -356,7 +348,7 @@ FENCE
 
 	b.Run("Hook no higlight", func(b *testing.B) {
 		files := filesTemplate + `
--- layouts/_default/_markup/render-codeblock.html --
+-- layouts/_markup/render-codeblock.html --
 {{ .Inner }}
 `
 
@@ -371,10 +363,10 @@ func TestHookInfiniteRecursion(t *testing.T) {
 	for _, renderFunc := range []string{"markdownify", ".Page.RenderString"} {
 		t.Run(renderFunc, func(t *testing.T) {
 			files := `
--- config.toml --
--- layouts/_default/_markup/render-link.html --
+-- hugo.toml --
+-- layouts/_markup/render-link.html --
 <a href="{{ .Destination | safeURL }}">{{ .Text | RENDERFUNC }}</a>
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 -- content/p1.md --
 ---
@@ -408,7 +400,7 @@ func TestQuotesInImgAltAttr(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark.extensions]
   typographer = false
 -- content/p1.md --
@@ -416,7 +408,7 @@ func TestQuotesInImgAltAttr(t *testing.T) {
 title: "p1"
 ---
 !["a"](b.jpg)
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -432,7 +424,7 @@ func TestLinkifyProtocol(t *testing.T) {
 
 	runTest := func(protocol string, withHook bool) *hugolib.IntegrationTestBuilder {
 		files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark]
 [markup.goldmark.extensions]
 linkify = true
@@ -445,13 +437,13 @@ Link no procol: www.example.org
 Link http procol: http://www.example.org
 Link https procol: https://www.example.org
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 		files = strings.ReplaceAll(files, "PROTOCOL", protocol)
 
 		if withHook {
-			files += `-- layouts/_default/_markup/render-link.html --
+			files += `-- layouts/_markup/render-link.html --
 <a href="{{ .Destination | safeURL }}">{{ .Text }}</a>`
 		}
 
@@ -496,7 +488,7 @@ func TestGoldmarkBugs(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark.renderer]
 unsafe = true
 -- content/p1.md --
@@ -513,7 +505,7 @@ a <!-- b --> c
 - This is a list item <!-- Comment: an innocent-looking comment -->
 
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -547,7 +539,7 @@ title: "p1"
 ![A's is > than B's](some-image.png)
 
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -566,7 +558,7 @@ func TestGoldmarkEmojiExtension(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 enableEmoji = true
 -- content/p1.md --
 ---
@@ -589,16 +581,16 @@ title: "p1"
 title: "p2"
 ---
 :heavy_check_mark:
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ $p := site.GetPage (.Get 0) }}
 {{ $p.RenderShortcodes }}
--- layouts/shortcodes/sc1.html --
+-- layouts/_shortcodes/sc1.html --
 sc1_begin|{{ .Inner }}|sc1_end
--- layouts/shortcodes/sc2.html --
+-- layouts/_shortcodes/sc2.html --
 sc2_begin|{{ .Inner | .Page.RenderString }}|sc2_end
--- layouts/shortcodes/sc3.html --
+-- layouts/_shortcodes/sc3.html --
 sc3_begin|{{ .Inner }}|sc3_end
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -622,14 +614,14 @@ func TestEmojiDisabled(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 enableEmoji = false
 -- content/p1.md --
 ---
 title: "p1"
 ---
 :x:
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -647,7 +639,7 @@ func TestEmojiDefaultConfig(t *testing.T) {
 title: "p1"
 ---
 :x:
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -661,12 +653,12 @@ func TestGoldmarkTemplateDelims(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [minify]
   minifyOutput = true
 [minify.tdewolff.html]
   templateDelims = ["<?php","?>"]
--- layouts/index.html --
+-- layouts/home.html --
 <div class="foo">
 {{ safeHTML "<?php" }}
 echo "hello";
@@ -683,7 +675,7 @@ func TestPassthroughInlineFences(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark.extensions.passthrough]
 enable = true
 [markup.goldmark.extensions.passthrough.delimiters]
@@ -696,7 +688,7 @@ title: "p1"
 
 Inline equation that would be mangled by default parser: $a^*=x-b^*$
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -710,7 +702,7 @@ func TestPassthroughBlockFences(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark.extensions.passthrough]
 enable = true
 [markup.goldmark.extensions.passthrough.delimiters]
@@ -725,7 +717,7 @@ Block equation that would be mangled by default parser:
 
 $$a^*=x-b^*$$
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -739,7 +731,7 @@ func TestPassthroughWithAlternativeFences(t *testing.T) {
 	t.Parallel()
 
 	files := `
--- config.toml --
+-- hugo.toml --
 [markup.goldmark.extensions.passthrough]
 enable = true
 [markup.goldmark.extensions.passthrough.delimiters]
@@ -760,7 +752,7 @@ Block equation that would be mangled by default parser:
 a^*=x-b^*
 %!%
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -796,7 +788,7 @@ enable = false
 enable = false
 [markup.goldmark.extensions.extras.superscript]
 enable = false
--- layouts/index.html --
+-- layouts/home.html --
 {{ .Content }}
 -- content/_index.md --
 ---
@@ -847,14 +839,14 @@ markup.goldmark.renderer.unsafe = false
 title: "p1"
 ---
 <div>Some raw HTML</div>
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
 	b := hugolib.Test(t, files, hugolib.TestOptWarn())
 
 	b.AssertFileContent("public/p1/index.html", "<!-- raw HTML omitted -->")
-	b.AssertLogContains("WARN  Raw HTML omitted while rendering \"/content/p1.md\"; see https://gohugo.io/getting-started/configuration-markup/#rendererunsafe\nYou can suppress this warning by adding the following to your site configuration:\nignoreLogs = ['warning-goldmark-raw-html']")
+	b.AssertLogContains("WARN  Raw HTML omitted while rendering \"/content/p1.md\"; see https://gohugo.io/getting-started/configuration-markup/#rendererunsafe\nYou can suppress this warning by adding the following to your project configuration:\nignoreLogs = ['warning-goldmark-raw-html']")
 
 	b = hugolib.Test(t, strings.ReplaceAll(files, "markup.goldmark.renderer.unsafe = false", "markup.goldmark.renderer.unsafe = true"), hugolib.TestOptWarn())
 	b.AssertFileContent("public/p1/index.html", "! <!-- raw HTML omitted -->")
@@ -871,14 +863,14 @@ markup.goldmark.renderer.unsafe = false
 title: "p1"
 ---
 <em>raw HTML</em>
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
 	b := hugolib.Test(t, files, hugolib.TestOptWarn())
 
 	b.AssertFileContent("public/p1/index.html", "<!-- raw HTML omitted -->")
-	b.AssertLogContains("WARN  Raw HTML omitted while rendering \"/content/p1.md\"; see https://gohugo.io/getting-started/configuration-markup/#rendererunsafe\nYou can suppress this warning by adding the following to your site configuration:\nignoreLogs = ['warning-goldmark-raw-html']")
+	b.AssertLogContains("WARN  Raw HTML omitted while rendering \"/content/p1.md\"; see https://gohugo.io/getting-started/configuration-markup/#rendererunsafe\nYou can suppress this warning by adding the following to your project configuration:\nignoreLogs = ['warning-goldmark-raw-html']")
 
 	b = hugolib.Test(t, strings.ReplaceAll(files, "markup.goldmark.renderer.unsafe = false", "markup.goldmark.renderer.unsafe = true"), hugolib.TestOptWarn())
 	b.AssertFileContent("public/p1/index.html", "! <!-- raw HTML omitted -->")
@@ -897,7 +889,7 @@ title: "p1"
 ---
 # HTML comments
 
-## Simple 
+## Simple
 <!-- This is a comment -->
 
     <!-- This is a comment indented -->
@@ -918,7 +910,7 @@ title: "p1"
 <img border="0" src="pic_trulli.jpg" alt="Trulli">
 -->
 
-## XSS 
+## XSS
 
 <!-- --><script>alert("I just escaped the HTML comment")</script><!-- -->
 
@@ -931,15 +923,15 @@ This is a <!-- hidden--> word.
 
 This is a <!-- hidden --> word.
 
-This is a <!-- 
+This is a <!--
 hidden --> word.
 
-This is a <!-- 
+This is a <!--
 hidden
 --> word.
 
 
--- layouts/_default/single.html --
+-- layouts/single.html --
 {{ .Content }}
 `
 
@@ -961,3 +953,83 @@ hidden
 	)
 	b.AssertLogContains("! WARN")
 }
+
+func TestFootnoteExtension(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableKinds = ['home','rss','section','sitemap','taxonomy','term']
+[markup.goldmark.extensions.footnote]
+enable = false
+enableAutoIDPrefix = false
+TBD = 'back'
+-- layouts/all.html --
+{{ .Content }}
+-- content/p1.md --
+---
+title: p1
+---
+foo[^1] and bar[^2]
+
+[^1]: footnote one
+[^2]: footnote two
+-- content/p2.md --
+---
+title: p2
+---
+foo[^1] and bar[^2]
+
+[^1]: footnote one
+[^2]: footnote two
+-- content/_content.gotmpl --
+{{ range slice 3 4 }}
+  {{ $page := dict
+    "content" (dict "mediaType" "text/markdown" "value" "foo[^1] and bar[^2]\n\n[^1]: footnote one\n[^2]: footnote two")
+    "path" (printf "p%d" .)
+    "title" (printf "p%d" .)
+  }}
+  {{ $.AddPage $page }}
+{{ end }}
+`
+
+	// test enableAutoIDPrefix
+
+	want := "<p>foo[^1] and bar[^2]</p>\n<p>[^1]: footnote one\n[^2]: footnote two</p>"
+	b := hugolib.Test(t, files)
+	b.AssertFileContent("public/p1/index.html", want)
+	b.AssertFileContent("public/p2/index.html", want)
+	b.AssertFileContent("public/p3/index.html", want)
+	b.AssertFileContent("public/p4/index.html", want)
+
+	files = strings.ReplaceAll(files, "enable = false", "enable = true")
+	want = "<p>foo<sup id=\"fnref:1\"><a href=\"#fn:1\" class=\"footnote-ref\" role=\"doc-noteref\">1</a></sup> and bar<sup id=\"fnref:2\"><a href=\"#fn:2\" class=\"footnote-ref\" role=\"doc-noteref\">2</a></sup></p>\n<div class=\"footnotes\" role=\"doc-endnotes\">\n<hr>\n<ol>\n<li id=\"fn:1\">\n<p>footnote one&#160;<a href=\"#fnref:1\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n<li id=\"fn:2\">\n<p>footnote two&#160;<a href=\"#fnref:2\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n</ol>\n</div>"
+	b = hugolib.Test(t, files)
+	b.AssertFileContent("public/p1/index.html", want)
+	b.AssertFileContent("public/p2/index.html", want)
+	b.AssertFileContent("public/p3/index.html", want)
+	b.AssertFileContent("public/p4/index.html", want)
+
+	files = strings.ReplaceAll(files, "enableAutoIDPrefix = false", "enableAutoIDPrefix = true")
+	b = hugolib.Test(t, files)
+	b.AssertFileContent("public/p1/index.html",
+		"<p>foo<sup id=\"hb5cdcabc9e678612fnref:1\"><a href=\"#hb5cdcabc9e678612fn:1\" class=\"footnote-ref\" role=\"doc-noteref\">1</a></sup> and bar<sup id=\"hb5cdcabc9e678612fnref:2\"><a href=\"#hb5cdcabc9e678612fn:2\" class=\"footnote-ref\" role=\"doc-noteref\">2</a></sup></p>\n<div class=\"footnotes\" role=\"doc-endnotes\">\n<hr>\n<ol>\n<li id=\"hb5cdcabc9e678612fn:1\">\n<p>footnote one&#160;<a href=\"#hb5cdcabc9e678612fnref:1\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n<li id=\"hb5cdcabc9e678612fn:2\">\n<p>footnote two&#160;<a href=\"#hb5cdcabc9e678612fnref:2\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n</ol>\n</div>",
+	)
+	b.AssertFileContent("public/p2/index.html",
+		"<p>foo<sup id=\"h58e8265a0c07b195fnref:1\"><a href=\"#h58e8265a0c07b195fn:1\" class=\"footnote-ref\" role=\"doc-noteref\">1</a></sup> and bar<sup id=\"h58e8265a0c07b195fnref:2\"><a href=\"#h58e8265a0c07b195fn:2\" class=\"footnote-ref\" role=\"doc-noteref\">2</a></sup></p>\n<div class=\"footnotes\" role=\"doc-endnotes\">\n<hr>\n<ol>\n<li id=\"h58e8265a0c07b195fn:1\">\n<p>footnote one&#160;<a href=\"#h58e8265a0c07b195fnref:1\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n<li id=\"h58e8265a0c07b195fn:2\">\n<p>footnote two&#160;<a href=\"#h58e8265a0c07b195fnref:2\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n</ol>\n</div>",
+	)
+	b.AssertFileContent("public/p3/index.html",
+		"<p>foo<sup id=\"h0aab769290d7e233fnref:1\"><a href=\"#h0aab769290d7e233fn:1\" class=\"footnote-ref\" role=\"doc-noteref\">1</a></sup> and bar<sup id=\"h0aab769290d7e233fnref:2\"><a href=\"#h0aab769290d7e233fn:2\" class=\"footnote-ref\" role=\"doc-noteref\">2</a></sup></p>\n<div class=\"footnotes\" role=\"doc-endnotes\">\n<hr>\n<ol>\n<li id=\"h0aab769290d7e233fn:1\">\n<p>footnote one&#160;<a href=\"#h0aab769290d7e233fnref:1\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n<li id=\"h0aab769290d7e233fn:2\">\n<p>footnote two&#160;<a href=\"#h0aab769290d7e233fnref:2\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n</ol>\n</div>",
+	)
+	b.AssertFileContent("public/p4/index.html",
+		"<p>foo<sup id=\"ha35b794ad6e8626cfnref:1\"><a href=\"#ha35b794ad6e8626cfn:1\" class=\"footnote-ref\" role=\"doc-noteref\">1</a></sup> and bar<sup id=\"ha35b794ad6e8626cfnref:2\"><a href=\"#ha35b794ad6e8626cfn:2\" class=\"footnote-ref\" role=\"doc-noteref\">2</a></sup></p>\n<div class=\"footnotes\" role=\"doc-endnotes\">\n<hr>\n<ol>\n<li id=\"ha35b794ad6e8626cfn:1\">\n<p>footnote one&#160;<a href=\"#ha35b794ad6e8626cfnref:1\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n<li id=\"ha35b794ad6e8626cfn:2\">\n<p>footnote two&#160;<a href=\"#ha35b794ad6e8626cfnref:2\" class=\"footnote-backref\" role=\"doc-backlink\">&#x21a9;&#xfe0e;</a></p>\n</li>\n</ol>\n</div>",
+	)
+
+	// test backlinkHTML
+
+	files = strings.ReplaceAll(files, "TBD", "backlinkHTML")
+	b = hugolib.Test(t, files)
+	b.AssertFileContent("public/p1/index.html",
+		"<p>foo<sup id=\"hb5cdcabc9e678612fnref:1\"><a href=\"#hb5cdcabc9e678612fn:1\" class=\"footnote-ref\" role=\"doc-noteref\">1</a></sup> and bar<sup id=\"hb5cdcabc9e678612fnref:2\"><a href=\"#hb5cdcabc9e678612fn:2\" class=\"footnote-ref\" role=\"doc-noteref\">2</a></sup></p>\n<div class=\"footnotes\" role=\"doc-endnotes\">\n<hr>\n<ol>\n<li id=\"hb5cdcabc9e678612fn:1\">\n<p>footnote one&#160;<a href=\"#hb5cdcabc9e678612fnref:1\" class=\"footnote-backref\" role=\"doc-backlink\">back</a></p>\n</li>\n<li id=\"hb5cdcabc9e678612fn:2\">\n<p>footnote two&#160;<a href=\"#hb5cdcabc9e678612fnref:2\" class=\"footnote-backref\" role=\"doc-backlink\">back</a></p>\n</li>\n</ol>\n</div>",
+	)
+}
