@@ -33,7 +33,6 @@ const (
 	lineNosKey     = "linenos"
 	hlLinesKey     = "hl_lines"
 	linosStartKey  = "linenostart"
-	noHlKey        = "nohl"
 )
 
 var DefaultConfig = Config{
@@ -45,21 +44,23 @@ var DefaultConfig = Config{
 	NoClasses:          true,
 	LineNumbersInTable: true,
 	TabWidth:           4,
+	WrapperClass:       "highlight",
 }
 
 type Config struct {
 	Style string
 
+	// Enable syntax highlighting of fenced code blocks.
 	CodeFences bool
 
+	// The class or classes to use for the outermost element of the highlighted code.
+	WrapperClass string
+
 	// Use inline CSS styles.
 	NoClasses bool
 
-	// No highlighting.
-	NoHl bool
-
 	// When set, line numbers will be printed.
-	LineNos            bool
+	LineNos            any
 	LineNumbersInTable bool
 
 	// When set, add links to line numbers
@@ -84,16 +85,44 @@ type Config struct {
 	GuessSyntax bool
 }
 
-func (cfg Config) toHTMLOptions() []html.Option {
-	var lineAnchors string
+const errLineNosMsg = `lineNos must be one of true, false, "inline", or "table"; got %[1]v (%[1]T)`
+
+func (cfg Config) toHTMLOptions() ([]html.Option, error) {
+	var (
+		lineAnchors string
+		lineNos     bool
+		inTable     = cfg.LineNumbersInTable
+	)
+
 	if cfg.LineAnchors != "" {
 		lineAnchors = cfg.LineAnchors + "-"
 	}
+
+	switch v := cfg.LineNos.(type) {
+	case string:
+		switch v {
+		case "inline":
+			lineNos = true
+			inTable = false
+		case "table":
+			lineNos = true
+			inTable = true
+		default:
+			return nil, fmt.Errorf(errLineNosMsg, v)
+		}
+	case bool:
+		lineNos = v
+	case nil:
+		lineNos = false
+	default:
+		return nil, fmt.Errorf(errLineNosMsg, v)
+	}
+
 	options := []html.Option{
 		html.TabWidth(cfg.TabWidth),
-		html.WithLineNumbers(cfg.LineNos),
+		html.WithLineNumbers(lineNos),
 		html.BaseLineNumber(cfg.LineNoStart),
-		html.LineNumbersInTable(cfg.LineNumbersInTable),
+		html.LineNumbersInTable(inTable),
 		html.WithClasses(!cfg.NoClasses),
 		html.WithLinkableLineNumbers(cfg.AnchorLineNos, lineAnchors),
 		html.InlineCode(cfg.Hl_inline),
@@ -116,7 +145,7 @@ func (cfg Config) toHTMLOptions() []html.Option {
 		}
 	}
 
-	return options
+	return options, nil
 }
 
 func applyOptions(opts any, cfg *Config) error {
@@ -196,7 +225,7 @@ func parseHighlightOptions(in string) (map[string]any, error) {
 		return opts, nil
 	}
 
-	for _, v := range strings.Split(in, ",") {
+	for v := range strings.SplitSeq(in, ",") {
 		keyVal := strings.Split(v, "=")
 		key := strings.Trim(keyVal[0], " ")
 		if len(keyVal) != 2 {
@@ -229,8 +258,6 @@ func normalizeHighlightOptions(m map[string]any) {
 
 	for k, v := range m {
 		switch k {
-		case noHlKey:
-			m[noHlKey] = cast.ToBool(v)
 		case lineNosKey:
 			if v == "table" || v == "inline" {
 				m["lineNumbersInTable"] = v == "table"
@@ -266,8 +293,8 @@ func hlLinesToRanges(startLine int, s string) ([][2]int, error) {
 	// 1-2 3
 	// 1 3-4
 	// 1    3-4
-	fields := strings.Split(s, " ")
-	for _, field := range fields {
+	fields := strings.SplitSeq(s, " ")
+	for field := range fields {
 		field = strings.TrimSpace(field)
 		if field == "" {
 			continue
