@@ -19,13 +19,13 @@ import (
 	"image/color"
 	"strings"
 
+	"github.com/gohugoio/hugo/common/hmaps"
 	"github.com/gohugoio/hugo/common/hugio"
-	"github.com/gohugoio/hugo/common/maps"
 	"github.com/gohugoio/hugo/resources/resource"
 	"github.com/makeworld-the-better-one/dither/v2"
 	"github.com/mitchellh/mapstructure"
 
-	"github.com/disintegration/gift"
+	"github.com/gohugoio/gift"
 	"github.com/spf13/cast"
 )
 
@@ -36,10 +36,11 @@ type Filters struct{}
 
 // Process creates a filter that processes an image using the given specification.
 func (*Filters) Process(spec any) gift.Filter {
+	specs := strings.ToLower(cast.ToString(spec))
 	return filter{
-		Options: newFilterOpts(spec),
+		Options: newFilterOpts(specs),
 		Filter: processFilter{
-			spec: cast.ToString(spec),
+			spec: specs,
 		},
 	}
 }
@@ -78,12 +79,13 @@ func (*Filters) Text(text string, options ...any) gift.Filter {
 		x:           10,
 		y:           10,
 		alignx:      "left",
+		aligny:      "top",
 		linespacing: 2,
 	}
 
-	var opt maps.Params
+	var opt hmaps.Params
 	if len(options) > 0 {
-		opt = maps.MustToParamsAndPrepare(options[0])
+		opt = hmaps.MustToParamsAndPrepare(options[0])
 		for option, v := range opt {
 			switch option {
 			case "color":
@@ -101,6 +103,11 @@ func (*Filters) Text(text string, options ...any) gift.Filter {
 				if tf.alignx != "left" && tf.alignx != "center" && tf.alignx != "right" {
 					panic("alignx must be one of left, center, right")
 				}
+			case "aligny":
+				tf.aligny = cast.ToString(v)
+				if tf.aligny != "top" && tf.aligny != "center" && tf.aligny != "bottom" {
+					panic("aligny must be one of top, center, bottom")
+				}
 
 			case "linespacing":
 				tf.linespacing = cast.ToInt(v)
