@@ -20,12 +20,14 @@ import (
 	"fmt"
 	"html/template"
 
+	"github.com/gohugoio/hugo/hugolib/roles"
+	"github.com/gohugoio/hugo/hugolib/sitesmatrix"
 	"github.com/gohugoio/hugo/markup/converter"
 	"github.com/gohugoio/hugo/markup/tableofcontents"
 
 	"github.com/gohugoio/hugo/config"
 
-	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/common/hstore"
 	"github.com/gohugoio/hugo/common/paths"
 	"github.com/gohugoio/hugo/compare"
 
@@ -185,14 +187,16 @@ type PageMetaResource interface {
 	resource.Resource
 }
 
+type PageMetaLanguageResource interface {
+	PageMetaResource
+	resource.LanguageProvider
+}
+
 // PageMetaProvider provides page metadata, typically provided via front matter.
 type PageMetaProvider interface {
 	// The 4 page dates
 	resource.Dated
 
-	// Aliases forms the base for redirects generation.
-	Aliases() []string
-
 	// BundleType returns the bundle type: `leaf`, `branch` or an empty string.
 	BundleType() string
 
@@ -224,9 +228,6 @@ type PageMetaProvider interface {
 	// IsPage returns whether this is a regular content
 	IsPage() bool
 
-	// Param looks for a param in Page and then in Site config.
-	Param(key any) (any, error)
-
 	// Path gets the relative path, including file name and extension if relevant,
 	// to the source of this Page. It will be relative to any content root.
 	Path() string
@@ -234,9 +235,6 @@ type PageMetaProvider interface {
 	// The slug, typically defined in front matter.
 	Slug() string
 
-	// This page's language code. Will be the same as the site's.
-	Lang() string
-
 	// IsSection returns whether this is a section
 	IsSection() bool
 
@@ -260,7 +258,7 @@ type PageMetaProvider interface {
 // This is currently only used to generate keywords for related content.
 // If nameLower is not one of the metadata interface methods, we
 // look in Params.
-func NamedPageMetaValue(p PageMetaResource, nameLower string) (any, bool, error) {
+func NamedPageMetaValue(p PageMetaLanguageResource, nameLower string) (any, bool, error) {
 	var (
 		v   any
 		err error
@@ -277,8 +275,6 @@ func NamedPageMetaValue(p PageMetaResource, nameLower string) (any, bool, error)
 		v = p.Section()
 	case "lang":
 		v = p.Lang()
-	case "aliases":
-		v = p.Aliases()
 	case "name":
 		v = p.Name()
 	case "keywords":
@@ -344,7 +340,11 @@ type PageWithoutContent interface {
 	RenderShortcodesProvider
 	resource.Resource
 	PageMetaProvider
+
+	Param(key any) (any, error)
+	Aliases() []string
 	PageMetaInternalProvider
+
 	resource.LanguageProvider
 
 	// For pages backed by a file.
@@ -374,6 +374,7 @@ type PageWithoutContent interface {
 	resource.TranslationKeyProvider
 	TranslationsProvider
 
+	SiteProvider
 	SitesProvider
 
 	// Helper methods
@@ -384,9 +385,9 @@ type PageWithoutContent interface {
 	// Note that this Scratch gets reset on server rebuilds. See Store() for a variant that survives.
 	// Scratch returns a "scratch pad" that can be used to store state.
 	// Deprecated: From Hugo v0.138.0 this is just an alias for Store.
-	Scratch() *maps.Scratch
+	Scratch() *hstore.Scratch
 
-	maps.StoreProvider
+	hstore.StoreProvider
 
 	RelatedKeywordsProvider
 
@@ -400,18 +401,16 @@ type PageWithoutContent interface {
 	HeadingsFiltered(context.Context) tableofcontents.Headings
 }
 
+type SiteDimensionProvider interface {
+	Role() roles.Role
+}
+
 // Positioner provides next/prev navigation.
 type Positioner interface {
 	// Next points up to the next regular page (sorted by Hugo’s default sort).
 	Next() Page
 	// Prev points down to the previous regular page (sorted by Hugo’s default sort).
 	Prev() Page
-
-	// Deprecated: Use Prev. Will be removed in Hugo 0.57
-	PrevPage() Page
-
-	// Deprecated: Use Next. Will be removed in Hugo 0.57
-	NextPage() Page
 }
 
 // RawContentProvider provides the raw, unprocessed content of the page.
@@ -455,14 +454,32 @@ type ShortcodeInfoProvider interface {
 	HasShortcode(name string) bool
 }
 
+type nopHugoSitesProvider struct{}
+
+func (nopHugoSitesProvider) Sites() Sites {
+	return nil
+}
+
+func (nopHugoSitesProvider) Data() map[string]any {
+	return nil
+}
+
 // SitesProvider provide accessors to get sites.
 type SitesProvider interface {
-	// Site returns the current site.
-	Site() Site
-	// Sites returns all sites.
+	// Sites returns all sites for all dimensions.
 	Sites() Sites
 }
 
+// DataProvider provides access to the data directory.
+type DataProvider interface {
+	Data() map[string]any
+}
+
+// SiteProvider provides access to the current site.
+type SiteProvider interface {
+	Site() Site
+}
+
 // TableOfContentsProvider provides the table of contents for a Page.
 type TableOfContentsProvider interface {
 	// TableOfContents returns the table of contents for the page rendered as HTML.
@@ -531,6 +548,19 @@ type TreeProvider interface {
 	SectionsPath() string
 }
 
+// SiteVectorProvider provides the dimensions of a Page.
+type SiteVectorProvider interface {
+	SiteVector() sitesmatrix.Vector
+}
+
+// GetSiteVector returns the site vector for a Page.
+func GetSiteVector(p Page) sitesmatrix.Vector {
+	if sp, ok := p.(SiteVectorProvider); ok {
+		return sp.SiteVector()
+	}
+	return sitesmatrix.Vector{}
+}
+
 // PageWithContext is a Page with a context.Context.
 type PageWithContext struct {
 	Page
