@@ -21,7 +21,7 @@ import (
 	"html/template"
 	"time"
 
-	"github.com/gohugoio/hugo/hugofs/files"
+	"github.com/gohugoio/hugo/hugolib/roles"
 	"github.com/gohugoio/hugo/markup/converter"
 	"github.com/gohugoio/hugo/markup/tableofcontents"
 
@@ -29,8 +29,8 @@ import (
 
 	"github.com/gohugoio/hugo/navigation"
 
-	"github.com/gohugoio/hugo/common/hugo"
-	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/common/hmaps"
+	"github.com/gohugoio/hugo/common/hstore"
 	"github.com/gohugoio/hugo/common/paths"
 	"github.com/gohugoio/hugo/source"
 
@@ -59,12 +59,6 @@ var (
 // PageNop implements Page, but does nothing.
 type nopPage int
 
-var noOpPathInfo = media.DefaultPathParser.Parse(files.ComponentFolderContent, "no-op.md")
-
-func (p *nopPage) Err() resource.ResourceError {
-	return nil
-}
-
 func (p *nopPage) Aliases() []string {
 	return nil
 }
@@ -185,8 +179,8 @@ func (p *nopPage) GetTerms(taxonomy string) Pages {
 	return nil
 }
 
-func (p *nopPage) GitInfo() source.GitInfo {
-	return source.GitInfo{}
+func (p *nopPage) GitInfo() *source.GitInfo {
+	return nil
 }
 
 func (p *nopPage) CodeOwners() []string {
@@ -201,7 +195,7 @@ func (p *nopPage) HasShortcode(name string) bool {
 	return false
 }
 
-func (p *nopPage) Hugo() (h hugo.HugoInfo) {
+func (p *nopPage) Hugo() (h HugoInfo) {
 	return
 }
 
@@ -261,6 +255,10 @@ func (p *nopPage) Language() *langs.Language {
 	return nil
 }
 
+func (p *nopPage) Role() roles.Role {
+	return nil
+}
+
 func (p *nopPage) Lastmod() (t time.Time) {
 	return
 }
@@ -321,7 +319,7 @@ func (p *nopPage) Param(key any) (any, error) {
 	return nil, nil
 }
 
-func (p *nopPage) Params() maps.Params {
+func (p *nopPage) Params() hmaps.Params {
 	return nil
 }
 
@@ -342,7 +340,7 @@ func (p *nopPage) Path() string {
 }
 
 func (p *nopPage) PathInfo() *paths.Path {
-	return noOpPathInfo
+	return nil
 }
 
 func (p *nopPage) Permalink() string {
@@ -373,14 +371,6 @@ func (p *nopPage) NextInSection() Page {
 	return nil
 }
 
-func (p *nopPage) PrevPage() Page {
-	return nil
-}
-
-func (p *nopPage) NextPage() Page {
-	return nil
-}
-
 func (p *nopPage) RawContent() string {
 	return ""
 }
@@ -421,11 +411,11 @@ func (p *nopPage) Resources() resource.Resources {
 	return nil
 }
 
-func (p *nopPage) Scratch() *maps.Scratch {
+func (p *nopPage) Scratch() *hstore.Scratch {
 	return nil
 }
 
-func (p *nopPage) Store() *maps.Scratch {
+func (p *nopPage) Store() *hstore.Scratch {
 	return nil
 }
 
