@@ -19,7 +19,6 @@ import (
 	"math"
 	"reflect"
 
-	"github.com/gohugoio/hugo/common/hugo"
 	"github.com/gohugoio/hugo/config"
 
 	"github.com/spf13/cast"
@@ -194,13 +193,6 @@ func (p *Paginator) Pagers() pagers {
 	return p.pagers
 }
 
-// PageSize returns the size of each paginator page.
-// Deprecated: Use PagerSize instead.
-func (p *Paginator) PageSize() int {
-	hugo.Deprecate("PageSize", "Use PagerSize instead.", "v0.128.0")
-	return p.size
-}
-
 // PagerSize returns the size of each paginator page.
 func (p *Paginator) PagerSize() int {
 	return p.size
