@@ -14,17 +14,19 @@
 package page
 
 import (
-	"html/template"
+	"slices"
 	"time"
 
-	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/common/hmaps"
+	"github.com/gohugoio/hugo/common/hstore"
 	"github.com/gohugoio/hugo/config/privacy"
 	"github.com/gohugoio/hugo/config/services"
+	"github.com/gohugoio/hugo/hugolib/roles"
+	"github.com/gohugoio/hugo/hugolib/versions"
 	"github.com/gohugoio/hugo/identity"
 
 	"github.com/gohugoio/hugo/config"
 
-	"github.com/gohugoio/hugo/common/hugo"
 	"github.com/gohugoio/hugo/langs"
 	"github.com/gohugoio/hugo/navigation"
 )
@@ -34,6 +36,15 @@ type Site interface {
 	// Returns the Language configured for this Site.
 	Language() *langs.Language
 
+	// Returns the Site in one of dimensions language, version or role.
+	Dimension(string) SiteDimension
+
+	// Returns the role configured for this Site.
+	Role() roles.Role
+
+	// Returns the version configured for this Site.
+	Version() versions.Version
+
 	// Returns all the languages configured for all sites.
 	Languages() langs.Languages
 
@@ -54,9 +65,6 @@ type Site interface {
 	// A shortcut to the home
 	Home() Page
 
-	// Deprecated: Use hugo.IsServer instead.
-	IsServer() bool
-
 	// Returns the server port.
 	ServerPort() int
 
@@ -69,14 +77,17 @@ type Site interface {
 	// Returns the configured copyright information for this Site.
 	Copyright() string
 
-	// Returns all Sites for all languages.
+	// Returns all sites for all dimensions.
 	Sites() Sites
 
 	// Returns Site currently rendering.
 	Current() Site
 
+	// Reports whether this site is the default across all dimensions.
+	IsDefault() bool
+
 	// Returns a struct with some information about the build.
-	Hugo() hugo.HugoInfo
+	Hugo() HugoInfo
 
 	// Returns the BaseURL for this Site.
 	BaseURL() string
@@ -84,9 +95,6 @@ type Site interface {
 	// Returns a taxonomy map.
 	Taxonomies() TaxonomyList
 
-	// Deprecated: Use .Lastmod instead.
-	LastChange() time.Time
-
 	// Returns the last modification date of the content.
 	Lastmod() time.Time
 
@@ -97,7 +105,7 @@ type Site interface {
 	MainSections() []string
 
 	// Returns the Params configured for this site.
-	Params() maps.Params
+	Params() hmaps.Params
 
 	// Param is a convenience method to do lookups in Params.
 	Param(key any) (any, error)
@@ -105,35 +113,20 @@ type Site interface {
 	// Returns a map of all the data inside /data.
 	Data() map[string]any
 
-	// Returns the site config.
+	// Returns the project config.
 	Config() SiteConfig
 
-	// Deprecated: Use taxonomies instead.
-	Author() map[string]interface{}
-
-	// Deprecated: Use taxonomies instead.
-	Authors() AuthorList
-
-	// Deprecated: Use .Site.Params instead.
-	Social() map[string]string
-
-	// Deprecated: Use Config().Services.GoogleAnalytics instead.
-	GoogleAnalytics() string
-
-	// Deprecated: Use Config().Privacy.Disqus instead.
-	DisqusShortname() string
-
 	// BuildDrafts is deprecated and will be removed in a future release.
 	BuildDrafts() bool
 
-	// Deprecated: Use hugo.IsMultilingual instead.
-	IsMultiLingual() bool
-
 	// LanguagePrefix returns the language prefix for this site.
 	LanguagePrefix() string
 
-	// Deprecated: Use .Site.Home.OutputFormats.Get "rss" instead.
-	RSSLink() template.URL
+	hstore.StoreProvider
+
+	// String returns a string representation of the site.
+	// Note that this representation may change in the future.
+	String() string
 
 	// For internal use only.
 	// This will panic if the site is not fully initialized.
@@ -142,22 +135,22 @@ type Site interface {
 	CheckReady()
 }
 
+// SiteDimension represents a dimension of the site.
+type SiteDimension interface {
+	Name() string
+}
+
 // Sites represents an ordered list of sites (languages).
 type Sites []Site
 
-// Deprecated: Use .Sites.Default instead.
-func (s Sites) First() Site {
-	hugo.Deprecate(".Sites.First", "Use .Sites.Default instead.", "v0.127.0")
-	return s.Default()
-}
-
-// Default is a convenience method to get the site corresponding to the default
-// content language.
+// Default is a convenience method to get the default site.
 func (s Sites) Default() Site {
-	if len(s) == 0 {
-		return nil
+	if idx := slices.IndexFunc(s, func(ss Site) bool {
+		return ss.IsDefault()
+	}); idx != -1 {
+		return s[idx]
 	}
-	return s[0]
+	return nil
 }
 
 // Some additional interfaces implemented by siteWrapper that's not on Site.
@@ -178,26 +171,6 @@ func (s *siteWrapper) Key() string {
 	return s.s.Language().Lang
 }
 
-// Deprecated: Use .Site.Params instead.
-func (s *siteWrapper) Social() map[string]string {
-	return s.s.Social()
-}
-
-// Deprecated: Use taxonomies instead.
-func (s *siteWrapper) Author() map[string]interface{} {
-	return s.s.Author()
-}
-
-// Deprecated: Use taxonomies instead.
-func (s *siteWrapper) Authors() AuthorList {
-	return s.s.Authors()
-}
-
-// Deprecated: Use .Site.Config.Services.GoogleAnalytics.ID instead.
-func (s *siteWrapper) GoogleAnalytics() string {
-	return s.s.GoogleAnalytics()
-}
-
 func (s *siteWrapper) GetPage(ref ...string) (Page, error) {
 	return s.s.GetPage(ref...)
 }
@@ -210,6 +183,18 @@ func (s *siteWrapper) Languages() langs.Languages {
 	return s.s.Languages()
 }
 
+func (s *siteWrapper) Role() roles.Role {
+	return s.s.Role()
+}
+
+func (s *siteWrapper) Dimension(d string) SiteDimension {
+	return s.s.Dimension(d)
+}
+
+func (s *siteWrapper) Version() versions.Version {
+	return s.s.Version()
+}
+
 func (s *siteWrapper) AllPages() Pages {
 	return s.s.AllPages()
 }
@@ -230,11 +215,6 @@ func (s *siteWrapper) Home() Page {
 	return s.s.Home()
 }
 
-// Deprecated: Use hugo.IsServer instead.
-func (s *siteWrapper) IsServer() bool {
-	return s.s.IsServer()
-}
-
 func (s *siteWrapper) ServerPort() int {
 	return s.s.ServerPort()
 }
@@ -259,11 +239,15 @@ func (s *siteWrapper) Current() Site {
 	return s.s.Current()
 }
 
+func (s *siteWrapper) IsDefault() bool {
+	return s.s.IsDefault()
+}
+
 func (s *siteWrapper) Config() SiteConfig {
 	return s.s.Config()
 }
 
-func (s *siteWrapper) Hugo() hugo.HugoInfo {
+func (s *siteWrapper) Hugo() HugoInfo {
 	return s.s.Hugo()
 }
 
@@ -275,11 +259,6 @@ func (s *siteWrapper) Taxonomies() TaxonomyList {
 	return s.s.Taxonomies()
 }
 
-// Deprecated: Use .Site.Lastmod instead.
-func (s *siteWrapper) LastChange() time.Time {
-	return s.s.LastChange()
-}
-
 func (s *siteWrapper) Lastmod() time.Time {
 	return s.s.Lastmod()
 }
@@ -292,7 +271,7 @@ func (s *siteWrapper) MainSections() []string {
 	return s.s.MainSections()
 }
 
-func (s *siteWrapper) Params() maps.Params {
+func (s *siteWrapper) Params() hmaps.Params {
 	return s.s.Params()
 }
 
@@ -308,23 +287,16 @@ func (s *siteWrapper) BuildDrafts() bool {
 	return s.s.BuildDrafts()
 }
 
-// Deprecated: Use hugo.IsMultilingual instead.
-func (s *siteWrapper) IsMultiLingual() bool {
-	return s.s.IsMultiLingual()
-}
-
-// Deprecated: Use .Site.Config.Services.Disqus.Shortname instead.
-func (s *siteWrapper) DisqusShortname() string {
-	return s.s.DisqusShortname()
-}
-
 func (s *siteWrapper) LanguagePrefix() string {
 	return s.s.LanguagePrefix()
 }
 
-// Deprecated: Use .Site.Home.OutputFormats.Get "rss" instead.
-func (s *siteWrapper) RSSLink() template.URL {
-	return s.s.RSSLink()
+func (s *siteWrapper) Store() *hstore.Scratch {
+	return s.s.Store()
+}
+
+func (s *siteWrapper) String() string {
+	return s.s.String()
 }
 
 // For internal use only.
@@ -338,26 +310,11 @@ func (s *siteWrapper) CheckReady() {
 }
 
 type testSite struct {
-	h hugo.HugoInfo
+	h HugoInfo
 	l *langs.Language
 }
 
-// Deprecated: Use taxonomies instead.
-func (s testSite) Author() map[string]interface{} {
-	return nil
-}
-
-// Deprecated: Use taxonomies instead.
-func (s testSite) Authors() AuthorList {
-	return AuthorList{}
-}
-
-// Deprecated: Use .Site.Params instead.
-func (s testSite) Social() map[string]string {
-	return make(map[string]string)
-}
-
-func (t testSite) Hugo() hugo.HugoInfo {
+func (t testSite) Hugo() HugoInfo {
 	return t.h
 }
 
@@ -365,11 +322,6 @@ func (t testSite) ServerPort() int {
 	return 1313
 }
 
-// Deprecated: Use .Site.Lastmod instead.
-func (testSite) LastChange() (t time.Time) {
-	return
-}
-
 func (testSite) Lastmod() (t time.Time) {
 	return
 }
@@ -402,6 +354,10 @@ func (t testSite) Current() Site {
 	return t
 }
 
+func (t testSite) IsDefault() bool {
+	return true
+}
+
 func (s testSite) LanguagePrefix() string {
 	return ""
 }
@@ -410,24 +366,26 @@ func (t testSite) Languages() langs.Languages {
 	return nil
 }
 
-// Deprecated: Use .Site.Config.Services.GoogleAnalytics.ID instead.
-func (t testSite) GoogleAnalytics() string {
-	return ""
+func (t testSite) Dimension(d string) SiteDimension {
+	return nil
 }
 
 func (t testSite) MainSections() []string {
 	return nil
 }
 
-// Deprecated: Use hugo.IsServer instead.
-func (t testSite) IsServer() bool {
-	return false
-}
-
 func (t testSite) Language() *langs.Language {
 	return t.l
 }
 
+func (t testSite) Role() roles.Role {
+	return nil
+}
+
+func (t testSite) Version() versions.Version {
+	return nil
+}
+
 func (t testSite) Home() Page {
 	return nil
 }
@@ -456,7 +414,7 @@ func (t testSite) BaseURL() string {
 	return ""
 }
 
-func (t testSite) Params() maps.Params {
+func (t testSite) Params() hmaps.Params {
 	return nil
 }
 
@@ -468,27 +426,20 @@ func (s testSite) Config() SiteConfig {
 	return SiteConfig{}
 }
 
-// Deprecated: Use .Site.Config.Services.Disqus.Shortname instead.
-func (testSite) DisqusShortname() string {
-	return ""
-}
-
 func (s testSite) BuildDrafts() bool {
 	return false
 }
 
-// Deprecated: Use hugo.IsMultilingual instead.
-func (s testSite) IsMultiLingual() bool {
-	return false
-}
-
 func (s testSite) Param(key any) (any, error) {
 	return nil, nil
 }
 
-// Deprecated: Use .Site.Home.OutputFormats.Get "rss" instead.
-func (s testSite) RSSLink() template.URL {
-	return ""
+func (s testSite) Store() *hstore.Scratch {
+	return hstore.NewScratch()
+}
+
+func (s testSite) String() string {
+	return "testSite"
 }
 
 func (s testSite) CheckReady() {
@@ -496,8 +447,11 @@ func (s testSite) CheckReady() {
 
 // NewDummyHugoSite creates a new minimal test site.
 func NewDummyHugoSite(conf config.AllProvider) Site {
+	opts := HugoInfoOptions{
+		Conf: conf,
+	}
 	return testSite{
-		h: hugo.NewInfo(conf, nil),
+		h: NewHugoInfo(opts),
 		l: &langs.Language{
 			Lang: "en",
 		},
