@@ -27,8 +27,8 @@ import (
 
 	"github.com/gohugoio/hugo/navigation"
 
-	"github.com/gohugoio/hugo/common/hugo"
-	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/common/hmaps"
+	"github.com/gohugoio/hugo/common/hstore"
 	"github.com/gohugoio/hugo/common/paths"
 	"github.com/gohugoio/hugo/config"
 	"github.com/gohugoio/hugo/hugofs"
@@ -74,7 +74,7 @@ func newTestPageWithFile(filename string) *testPage {
 		currentSection: &testPage{
 			sectionEntries: []string{"a", "b", "c"},
 		},
-		site: testSite{l: l},
+		site: &testSite{l: l},
 	}
 }
 
@@ -85,7 +85,7 @@ type testPage struct {
 	linkTitle   string
 	lang        string
 	section     string
-	site        testSite
+	site        *testSite
 
 	content string
 
@@ -247,7 +247,7 @@ func (p *testPage) HasShortcode(name string) bool {
 	panic("testpage: not implemented")
 }
 
-func (p *testPage) Hugo() hugo.HugoInfo {
+func (p *testPage) Hugo() HugoInfo {
 	panic("testpage: not implemented")
 }
 
@@ -397,7 +397,7 @@ func (p *testPage) Param(key any) (any, error) {
 	return resource.Param(p, nil, key)
 }
 
-func (p *testPage) Params() maps.Params {
+func (p *testPage) Params() hmaps.Params {
 	return p.params
 }
 
@@ -493,11 +493,11 @@ func (p *testPage) Resources() resource.Resources {
 	panic("testpage: not implemented")
 }
 
-func (p *testPage) Scratch() *maps.Scratch {
+func (p *testPage) Scratch() *hstore.Scratch {
 	panic("testpage: not implemented")
 }
 
-func (p *testPage) Store() *maps.Scratch {
+func (p *testPage) Store() *hstore.Scratch {
 	panic("testpage: not implemented")
 }
 
@@ -527,6 +527,9 @@ func (p *testPage) SectionsPath() string {
 }
 
 func (p *testPage) Site() Site {
+	if p.site == nil {
+		panic(fmt.Sprintf("testpage: site is nil for %q", p.path))
+	}
 	return p.site
 }
 
