@@ -23,7 +23,7 @@ import (
 	"testing"
 	"time"
 
-	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/common/hmaps"
 )
 
 func TestWhere(t *testing.T) {
@@ -176,35 +176,35 @@ func TestWhere(t *testing.T) {
 				{1: "a", 2: "m"},
 			},
 		},
-		// Case insensitive maps.Params
+		// Case insensitive hmaps.Params
 		// Slice of structs
 		{
-			seq: []TstParams{{params: maps.Params{"i": 0, "color": "indigo"}}, {params: maps.Params{"i": 1, "color": "blue"}}, {params: maps.Params{"i": 2, "color": "green"}}, {params: maps.Params{"i": 3, "color": "blue"}}},
+			seq: []TstParams{{params: hmaps.Params{"i": 0, "color": "indigo"}}, {params: hmaps.Params{"i": 1, "color": "blue"}}, {params: hmaps.Params{"i": 2, "color": "green"}}, {params: hmaps.Params{"i": 3, "color": "blue"}}},
 			key: ".Params.COLOR", match: "blue",
-			expect: []TstParams{{params: maps.Params{"i": 1, "color": "blue"}}, {params: maps.Params{"i": 3, "color": "blue"}}},
+			expect: []TstParams{{params: hmaps.Params{"i": 1, "color": "blue"}}, {params: hmaps.Params{"i": 3, "color": "blue"}}},
 		},
 		{
-			seq: []TstParams{{params: maps.Params{"nested": map[string]any{"color": "indigo"}}}, {params: maps.Params{"nested": map[string]any{"color": "blue"}}}},
+			seq: []TstParams{{params: hmaps.Params{"nested": map[string]any{"color": "indigo"}}}, {params: hmaps.Params{"nested": map[string]any{"color": "blue"}}}},
 			key: ".Params.NEsTED.COLOR", match: "blue",
-			expect: []TstParams{{params: maps.Params{"nested": map[string]any{"color": "blue"}}}},
+			expect: []TstParams{{params: hmaps.Params{"nested": map[string]any{"color": "blue"}}}},
 		},
 		{
-			seq: []TstParams{{params: maps.Params{"i": 0, "color": "indigo"}}, {params: maps.Params{"i": 1, "color": "blue"}}, {params: maps.Params{"i": 2, "color": "green"}}, {params: maps.Params{"i": 3, "color": "blue"}}},
+			seq: []TstParams{{params: hmaps.Params{"i": 0, "color": "indigo"}}, {params: hmaps.Params{"i": 1, "color": "blue"}}, {params: hmaps.Params{"i": 2, "color": "green"}}, {params: hmaps.Params{"i": 3, "color": "blue"}}},
 			key: ".Params", match: "blue",
 			expect: []TstParams{},
 		},
 		// Slice of maps
 		{
-			seq: []maps.Params{
+			seq: []hmaps.Params{
 				{"a": "a1", "b": "b1"}, {"a": "a2", "b": "b2"},
 			},
 			key: "B", match: "b2",
-			expect: []maps.Params{
+			expect: []hmaps.Params{
 				{"a": "a2", "b": "b2"},
 			},
 		},
 		{
-			seq: []maps.Params{
+			seq: []hmaps.Params{
 				{
 					"a": map[string]any{
 						"b": "b1",
@@ -217,7 +217,7 @@ func TestWhere(t *testing.T) {
 				},
 			},
 			key: "A.B", match: "b2",
-			expect: []maps.Params{
+			expect: []hmaps.Params{
 				{
 					"a": map[string]any{
 						"b": "b2",
@@ -622,14 +622,14 @@ func TestWhere(t *testing.T) {
 		},
 		{
 			seq: map[string]any{
-				"foo": []any{maps.Params{"a": 1, "b": 2}},
-				"bar": []any{maps.Params{"a": 3, "b": 4}},
-				"zap": []any{maps.Params{"a": 5, "b": 6}},
+				"foo": []any{hmaps.Params{"a": 1, "b": 2}},
+				"bar": []any{hmaps.Params{"a": 3, "b": 4}},
+				"zap": []any{hmaps.Params{"a": 5, "b": 6}},
 			},
 			key: "B", op: ">", match: 3,
 			expect: map[string]any{
-				"bar": []any{maps.Params{"a": 3, "b": 4}},
-				"zap": []any{maps.Params{"a": 5, "b": 6}},
+				"bar": []any{hmaps.Params{"a": 3, "b": 4}},
+				"zap": []any{hmaps.Params{"a": 5, "b": 6}},
 			},
 		},
 	} {
@@ -761,6 +761,7 @@ func TestCheckCondition(t *testing.T) {
 			expect{true, false},
 		},
 		{reflect.ValueOf(123), reflect.ValueOf([]int{45, 678}), "not in", expect{true, false}},
+		{reflect.ValueOf(123), reflect.ValueOf([]int{}), "not in", expect{true, false}},
 		{reflect.ValueOf("foo"), reflect.ValueOf([]string{"bar", "baz"}), "not in", expect{true, false}},
 		{
 			reflect.ValueOf(time.Date(2015, time.May, 26, 19, 18, 56, 12345, time.UTC)),
@@ -865,10 +866,10 @@ func BenchmarkWhereOps(b *testing.B) {
 	ns := newNs()
 	var seq []map[string]string
 	ctx := context.Background()
-	for i := 0; i < 500; i++ {
+	for range 500 {
 		seq = append(seq, map[string]string{"foo": "bar"})
 	}
-	for i := 0; i < 500; i++ {
+	for range 500 {
 		seq = append(seq, map[string]string{"foo": "baz"})
 	}
 	// Shuffle the sequence.
@@ -885,20 +886,40 @@ func BenchmarkWhereOps(b *testing.B) {
 	}
 
 	b.Run("eq", func(b *testing.B) {
-		for i := 0; i < b.N; i++ {
+		for b.Loop() {
 			runOps(b, "eq", "bar")
 		}
 	})
 
 	b.Run("ne", func(b *testing.B) {
-		for i := 0; i < b.N; i++ {
+		for b.Loop() {
 			runOps(b, "ne", "baz")
 		}
 	})
 
 	b.Run("like", func(b *testing.B) {
-		for i := 0; i < b.N; i++ {
+		for b.Loop() {
 			runOps(b, "like", "^bar")
 		}
 	})
 }
+
+func BenchmarkWhereSliceOfStructPointersWithMethod(b *testing.B) {
+	// TstRv2
+	ns := newNs()
+	seq := []*TstX{}
+
+	for i := range 1000 {
+		seq = append(seq, &TstX{A: "foo", B: "bar"})
+		if i%2 == 0 {
+			seq = append(seq, &TstX{A: "baz", B: "qux"})
+		}
+	}
+
+	for b.Loop() {
+		_, err := ns.Where(context.Background(), seq, "TstRv2", "eq", "bar")
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+}
