@@ -23,7 +23,6 @@ import (
 	"encoding/hex"
 	"fmt"
 	"hash"
-	"hash/fnv"
 
 	"github.com/spf13/cast"
 )
@@ -69,18 +68,6 @@ func (ns *Namespace) SHA256(v any) (string, error) {
 	return hex.EncodeToString(hash[:]), nil
 }
 
-// FNV32a hashes v using fnv32a algorithm.
-// <docsmeta>{"newIn": "0.98.0" }</docsmeta>
-func (ns *Namespace) FNV32a(v any) (int, error) {
-	conv, err := cast.ToStringE(v)
-	if err != nil {
-		return 0, err
-	}
-	algorithm := fnv.New32a()
-	algorithm.Write([]byte(conv))
-	return int(algorithm.Sum32()), nil
-}
-
 // HMAC returns a cryptographic hash that uses a key to sign a message.
 func (ns *Namespace) HMAC(h any, k any, m any, e ...any) (string, error) {
 	ha, err := cast.ToStringE(h)
