@@ -35,20 +35,53 @@ func init() {
 			Context: func(cctx context.Context, args ...any) (any, error) { return ctx, nil },
 		}
 
+		ns.AddMethodMapping(ctx.ByType,
+			nil,
+			[][2]string{},
+		)
+
+		ns.AddMethodMapping(ctx.Concat,
+			nil,
+			[][2]string{},
+		)
+
+		ns.AddMethodMapping(ctx.Copy,
+			nil,
+			[][2]string{},
+		)
+
+		ns.AddMethodMapping(ctx.ExecuteAsTemplate,
+			nil,
+			[][2]string{},
+		)
+
+		ns.AddMethodMapping(ctx.Fingerprint,
+			[]string{"fingerprint"},
+			[][2]string{},
+		)
+
+		ns.AddMethodMapping(ctx.FromString,
+			nil,
+			[][2]string{},
+		)
+
 		ns.AddMethodMapping(ctx.Get,
 			nil,
 			[][2]string{},
 		)
 
+		ns.AddMethodMapping(ctx.Match,
+			nil,
+			[][2]string{},
+		)
+
 		ns.AddMethodMapping(ctx.GetRemote,
 			nil,
 			[][2]string{},
 		)
 
-		// Add aliases for the most common transformations.
-
-		ns.AddMethodMapping(ctx.Fingerprint,
-			[]string{"fingerprint"},
+		ns.AddMethodMapping(ctx.Match,
+			nil,
 			[][2]string{},
 		)
 
@@ -57,18 +90,8 @@ func init() {
 			[][2]string{},
 		)
 
-		ns.AddMethodMapping(ctx.ToCSS,
-			[]string{"toCSS"},
-			[][2]string{},
-		)
-
-		ns.AddMethodMapping(ctx.PostCSS,
-			[]string{"postCSS"},
-			[][2]string{},
-		)
-
-		ns.AddMethodMapping(ctx.Babel,
-			[]string{"babel"},
+		ns.AddMethodMapping(ctx.PostProcess,
+			nil,
 			[][2]string{},
 		)
 
