@@ -20,9 +20,12 @@ import (
 	"strconv"
 	"sync/atomic"
 
+	"github.com/gohugoio/hugo/tpl/tplimpl"
+
 	"github.com/gohugoio/hugo/common/hashing"
 	"github.com/gohugoio/hugo/deps"
 	"github.com/gohugoio/hugo/tpl"
+	"github.com/gohugoio/hugo/tpl/partials"
 	"github.com/mitchellh/mapstructure"
 )
 
@@ -37,14 +40,15 @@ func New(deps *deps.Deps) *Namespace {
 
 // Namespace provides template functions for the "templates" namespace.
 type Namespace struct {
-	deps *deps.Deps
+	deps       *deps.Deps
+	partialsNs *partials.Namespace
 }
 
 // Exists returns whether the template with the given name exists.
 // Note that this is the Unix-styled relative path including filename suffix,
 // e.g. partials/header.html
 func (ns *Namespace) Exists(name string) bool {
-	return ns.deps.Tmpl().HasTemplate(name)
+	return ns.deps.GetTemplateStore().HasTemplate(name)
 }
 
 // Defer defers the execution of a template block.
@@ -70,6 +74,50 @@ type DeferOpts struct {
 	Data any
 }
 
+// Inner executes the inner content of a partial decorator.
+// Note that there is only one inner block per partial decorator, but inner may be called multiple times with, typically, different data.
+func (ns *Namespace) Inner(ctx context.Context, data any) (any, error) {
+	stack := tpl.Context.PartialDecoratorIDStack.Get(ctx)
+	id, ok := stack.Peek()
+	if !ok {
+		panic("no partial decorator ID on stack")
+	}
+
+	// Signal that inner exists.
+	id.Bool = true
+
+	partialName := fmt.Sprintf("%s%s", tplimpl.PartialDecoratorPrefix, id.Str)
+
+	v, err := ns.partialsNs.Include(ctx, partialName, data)
+
+	return v, err
+}
+
+// For internal use only.
+func (ns *Namespace) _PushPartialDecorator(ctx context.Context, id string) (any, error) {
+	tpl.Context.PartialDecoratorIDStack.Get(ctx).Push(&tpl.StringBool{Str: id, Bool: false})
+	return "", nil
+}
+
+// For internal use only.
+func (ns *Namespace) _PopPartialDecorator(ctx context.Context, id string) any {
+	stack := tpl.Context.PartialDecoratorIDStack.Get(ctx)
+	if stack == nil || stack.Len() == 0 {
+		panic("decorator stack is nil or empty")
+	}
+
+	// The stack is tied to the context, so no data race.
+	top, ok := stack.Pop()
+	if !ok || top.Str != id {
+		panic("partial decorator ID mismatch")
+	}
+	if !top.Bool {
+		// Prevents anything being rendered if inner was not called.
+		return ""
+	}
+	return top.Bool // return whether inner exists in the wrapped partial.
+}
+
 // DoDefer defers the execution of a template block.
 // For internal use only.
 func (ns *Namespace) DoDefer(ctx context.Context, id string, optsv any) string {
@@ -83,7 +131,7 @@ func (ns *Namespace) DoDefer(ctx context.Context, id string, optsv any) string {
 	templateName := id
 	var key string
 	if opts.Key != "" {
-		key = hashing.MD5FromStringHexEncoded(opts.Key)
+		key = hashing.XxHashFromStringHexEncoded(opts.Key)
 	} else {
 		key = strconv.FormatUint(defferedIDCounter.Add(1), 10)
 	}
@@ -93,7 +141,7 @@ func (ns *Namespace) DoDefer(ctx context.Context, id string, optsv any) string {
 	_, _ = ns.deps.BuildState.DeferredExecutions.Executions.GetOrCreate(id,
 		func() (*tpl.DeferredExecution, error) {
 			return &tpl.DeferredExecution{
-				TemplateName: templateName,
+				TemplatePath: templateName,
 				Ctx:          ctx,
 				Data:         opts.Data,
 				Executed:     false,
@@ -102,3 +150,8 @@ func (ns *Namespace) DoDefer(ctx context.Context, id string, optsv any) string {
 
 	return id
 }
+
+// Get information about the currently executing template.
+func (ns *Namespace) Current(ctx context.Context) *tpl.CurrentTemplateInfo {
+	return tpl.Context.CurrentTemplate.Get(ctx)
+}
